@@ -1,5 +1,6 @@
+from datetime import datetime
+
 import graphene
-from dagster.core.execution.retries import Retries
 from dagster.core.storage.pipeline_run import PipelineRunStatus, PipelineRunsFilter
 
 from .pipelines.status import GraphenePipelineRunStatus
@@ -28,6 +29,7 @@ class GraphenePipelineRunsFilter(graphene.InputObjectType):
     tags = graphene.List(graphene.NonNull(GrapheneExecutionTag))
     statuses = graphene.List(graphene.NonNull(GraphenePipelineRunStatus))
     snapshotId = graphene.Field(graphene.String)
+    updatedAfter = graphene.Field(graphene.String)
 
     class Meta:
         description = """This type represents a filter on pipeline runs.
@@ -49,12 +51,15 @@ class GraphenePipelineRunsFilter(graphene.InputObjectType):
         else:
             statuses = None
 
+        updated_after = datetime.fromisoformat(self.updatedAfter) if self.updatedAfter else None
+
         return PipelineRunsFilter(
             run_ids=self.runIds,
             pipeline_name=self.pipelineName,
             tags=tags,
             statuses=statuses,
             snapshot_id=self.snapshotId,
+            updated_after=updated_after,
         )
 
 
@@ -97,7 +102,7 @@ class GraphenePartitionSetSelector(graphene.InputObjectType):
         name = "PartitionSetSelector"
 
 
-class GraphenePartitionBackfillParams(graphene.InputObjectType):
+class GrapheneLaunchBackfillParams(graphene.InputObjectType):
     selector = graphene.NonNull(GraphenePartitionSetSelector)
     partitionNames = non_null_list(graphene.String)
     reexecutionSteps = graphene.List(graphene.NonNull(graphene.String))
@@ -106,7 +111,7 @@ class GraphenePartitionBackfillParams(graphene.InputObjectType):
     forceSynchronousSubmission = graphene.Boolean()
 
     class Meta:
-        name = "PartitionBackfillParams"
+        name = "LaunchBackfillParams"
 
 
 class GrapheneSensorSelector(graphene.InputObjectType):
@@ -201,26 +206,6 @@ class GrapheneStepExecution(graphene.InputObjectType):
         name = "StepExecution"
 
 
-class GrapheneRetriesPreviousAttempts(graphene.InputObjectType):
-    key = graphene.String()
-    count = graphene.Int()
-
-    class Meta:
-        name = "RetriesPreviousAttempts"
-
-
-class GrapheneRetries(graphene.InputObjectType):
-    mode = graphene.Field(graphene.String)
-    retries_previous_attempts = graphene.List(GrapheneRetriesPreviousAttempts)
-
-    class Meta:
-        name = "Retries"
-
-
-def create_retries_params(retries_config):
-    return Retries.from_graphql_input(retries_config)
-
-
 class GrapheneJobSelector(graphene.InputObjectType):
     class Meta:
         name = "JobSelector"
@@ -241,14 +226,11 @@ types = [
     GrapheneJobSelector,
     GrapheneMarshalledInput,
     GrapheneMarshalledOutput,
-    GraphenePartitionBackfillParams,
+    GrapheneLaunchBackfillParams,
     GraphenePartitionSetSelector,
     GraphenePipelineRunsFilter,
     GraphenePipelineSelector,
     GrapheneRepositorySelector,
-    GrapheneRetries,
-    GrapheneRetriesPreviousAttempts,
-    GrapheneRetriesPreviousAttempts,
     GrapheneScheduleSelector,
     GrapheneSensorSelector,
     GrapheneStepExecution,
