@@ -11,7 +11,7 @@ from dagster.core.host_representation.represented import RepresentedPipeline
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus, PipelineRunsFilter
 from dagster.core.storage.tags import TagType, get_tag_type
 
-from ...implementation.events import construct_basic_params
+from ...implementation.events import construct_basic_params, from_event_record
 from ...implementation.fetch_assets import get_assets_for_run_id
 from ...implementation.fetch_pipelines import get_pipeline_reference_or_raise
 from ...implementation.fetch_runs import get_run_by_id, get_runs, get_stats, get_step_stats
@@ -27,7 +27,12 @@ from ..errors import (
 )
 from ..execution import GrapheneExecutionPlan
 from ..logs.compute_logs import GrapheneComputeLogs
-from ..logs.events import GraphenePipelineRunStepStats, GrapheneStepMaterializationEvent
+from ..logs.events import (
+    GraphenePipelineRunEvent,
+    GraphenePipelineRunStepStats,
+    GrapheneStepMaterializationEvent,
+)
+from ..paging import GrapheneCursor
 from ..repository_origin import GrapheneRepositoryOrigin
 from ..schedules.schedules import GrapheneSchedule
 from ..sensors import GrapheneSensor
@@ -76,16 +81,12 @@ class GrapheneAssetMaterialization(graphene.ObjectType):
 
 
 class GrapheneAsset(graphene.ObjectType):
+    id = graphene.NonNull(graphene.String)
     key = graphene.NonNull(GrapheneAssetKey)
     assetMaterializations = graphene.Field(
         non_null_list(GrapheneAssetMaterialization),
         partitions=graphene.List(graphene.String),
-        cursor=graphene.String(),
-        limit=graphene.Int(),
-    )
-    runs = graphene.Field(
-        non_null_list(lambda: GraphenePipelineRun),
-        cursor=graphene.String(),
+        beforeTimestampMillis=graphene.String(),
         limit=graphene.Int(),
     )
     tags = non_null_list(GrapheneAssetTag)
@@ -98,46 +99,29 @@ class GrapheneAsset(graphene.ObjectType):
         check.opt_dict_param(tags, "tags", key_type=str, value_type=str)
         self._tags = tags
 
+    def resolve_id(self, _):
+        return self.key
+
     def resolve_assetMaterializations(self, graphene_info, **kwargs):
         from ...implementation.fetch_assets import get_asset_events
 
+        try:
+            before_timestamp = (
+                int(kwargs.get("beforeTimestampMillis")) / 1000.0
+                if kwargs.get("beforeTimestampMillis")
+                else None
+            )
+        except ValueError:
+            before_timestamp = None
+
         return [
             GrapheneAssetMaterialization(event=event)
             for event in get_asset_events(
                 graphene_info,
                 self.key,
                 kwargs.get("partitions"),
-                kwargs.get("cursor"),
-                kwargs.get("limit"),
-            )
-        ]
-
-    def resolve_runs(self, graphene_info, **kwargs):
-        from ...implementation.fetch_assets import get_asset_run_ids
-
-        cursor = kwargs.get("cursor")
-        limit = kwargs.get("limit")
-
-        run_ids = get_asset_run_ids(graphene_info, self.key)
-
-        if not run_ids:
-            return []
-
-        # for now, handle cursor/limit here instead of in the DB layer
-        if cursor:
-            try:
-                idx = run_ids.index(cursor)
-                run_ids = run_ids[idx:]
-            except ValueError:
-                return []
-
-        if limit:
-            run_ids = run_ids[:limit]
-
-        return [
-            GraphenePipelineRun(r)
-            for r in graphene_info.context.instance.get_runs(
-                filters=PipelineRunsFilter(run_ids=run_ids)
+                before_timestamp=before_timestamp,
+                limit=kwargs.get("limit"),
             )
         ]
 
@@ -184,6 +168,10 @@ class GraphenePipelineRun(graphene.ObjectType):
     parentRunId = graphene.Field(graphene.String)
     canTerminate = graphene.NonNull(graphene.Boolean)
     assets = non_null_list(GrapheneAsset)
+    events = graphene.Field(
+        non_null_list(GraphenePipelineRunEvent),
+        after=graphene.Argument(GrapheneCursor),
+    )
 
     class Meta:
         name = "PipelineRun"
@@ -288,6 +276,10 @@ class GraphenePipelineRun(graphene.ObjectType):
     def resolve_assets(self, graphene_info):
         return get_assets_for_run_id(graphene_info, self.run_id)
 
+    def resolve_events(self, graphene_info, after=-1):
+        events = graphene_info.context.instance.logs_after(self.run_id, cursor=after)
+        return [from_event_record(event, self._pipeline_run.pipeline_name) for event in events]
+
 
 class GrapheneIPipelineSnapshotMixin:
     # Mixin this class to implement IPipelineSnapshot
