@@ -7,36 +7,36 @@ from ...implementation.execution import (
     delete_pipeline_run,
     launch_pipeline_execution,
     launch_pipeline_reexecution,
+    resume_partition_backfill,
     terminate_pipeline_execution,
-    wipe_asset,
-)
-from ...implementation.external import (
-    fetch_repository_locations,
-    get_full_external_pipeline_or_raise,
+    wipe_assets,
 )
+from ...implementation.external import fetch_workspace, get_full_external_pipeline_or_raise
 from ...implementation.utils import (
     ExecutionMetadata,
     ExecutionParams,
     UserFacingGraphQLError,
     capture_error,
+    check_read_only,
     pipeline_selector_from_graphql,
 )
 from ..asset_key import GrapheneAssetKey
-from ..backfill import GrapheneCancelBackfillResult, GrapheneLaunchBackfillResult
+from ..backfill import (
+    GrapheneCancelBackfillResult,
+    GrapheneLaunchBackfillResult,
+    GrapheneResumeBackfillResult,
+)
 from ..errors import (
     GrapheneAssetNotFoundError,
     GrapheneConflictingExecutionParamsError,
     GraphenePipelineRunNotFoundError,
     GraphenePresetNotFoundError,
     GraphenePythonError,
+    GrapheneReadOnlyError,
     GrapheneReloadNotSupported,
     GrapheneRepositoryLocationNotFound,
 )
-from ..external import (
-    GrapheneRepositoryLocation,
-    GrapheneRepositoryLocationConnection,
-    GrapheneRepositoryLocationLoadFailure,
-)
+from ..external import GrapheneWorkspace, GrapheneWorkspaceLocationEntry
 from ..inputs import GrapheneAssetKeyInput, GrapheneExecutionParams, GrapheneLaunchBackfillParams
 from ..pipelines.pipeline import GraphenePipelineRun
 from ..runs import GrapheneLaunchPipelineExecutionResult, GrapheneLaunchPipelineReexecutionResult
@@ -46,6 +46,7 @@ from ..schedules import (
     GrapheneStopRunningScheduleMutation,
 )
 from ..sensors import GrapheneStartSensorMutation, GrapheneStopSensorMutation
+from ..util import non_null_list
 
 
 def create_execution_params(graphene_info, graphql_execution_params):
@@ -127,6 +128,7 @@ class GrapheneDeletePipelineRunResult(graphene.Union):
     class Meta:
         types = (
             GrapheneDeletePipelineRunSuccess,
+            GrapheneReadOnlyError,
             GraphenePythonError,
             GraphenePipelineRunNotFoundError,
         )
@@ -142,6 +144,8 @@ class GrapheneDeleteRunMutation(graphene.Mutation):
     class Meta:
         name = "DeleteRunMutation"
 
+    @capture_error
+    @check_read_only
     def mutate(self, graphene_info, **kwargs):
         run_id = kwargs["runId"]
         return delete_pipeline_run(graphene_info, run_id)
@@ -168,6 +172,7 @@ class GrapheneTerminatePipelineExecutionResult(graphene.Union):
             GrapheneTerminatePipelineExecutionSuccess,
             GrapheneTerminatePipelineExecutionFailure,
             GraphenePipelineRunNotFoundError,
+            GrapheneReadOnlyError,
             GraphenePythonError,
         )
         name = "TerminatePipelineExecutionResult"
@@ -194,6 +199,8 @@ class GrapheneLaunchPipelineExecutionMutation(graphene.Mutation):
         description = "Launch a pipeline run via the run launcher configured on the instance."
         name = "LaunchPipelineExecutionMutation"
 
+    @capture_error
+    @check_read_only
     def mutate(self, graphene_info, **kwargs):
         return create_execution_params_and_launch_pipeline_exec(
             graphene_info, kwargs["executionParams"]
@@ -210,6 +217,8 @@ class GrapheneLaunchBackfillMutation(graphene.Mutation):
         description = "Launches a set of partition backfill runs via the run launcher configured on the instance."
         name = "LaunchBackfillMutation"
 
+    @capture_error
+    @check_read_only
     def mutate(self, graphene_info, **kwargs):
         return create_and_launch_partition_backfill(graphene_info, kwargs["backfillParams"])
 
@@ -224,10 +233,28 @@ class GrapheneCancelBackfillMutation(graphene.Mutation):
         description = "Marks a partition backfill as canceled."
         name = "CancelBackfillMutation"
 
+    @capture_error
+    @check_read_only
     def mutate(self, graphene_info, **kwargs):
         return cancel_partition_backfill(graphene_info, kwargs["backfillId"])
 
 
+class GrapheneResumeBackfillMutation(graphene.Mutation):
+    Output = graphene.NonNull(GrapheneResumeBackfillResult)
+
+    class Arguments:
+        backfillId = graphene.NonNull(graphene.String)
+
+    class Meta:
+        description = "Retries a set of partition backfill runs via the run launcher configured on the instance."
+        name = "ResumeBackfillMutation"
+
+    @capture_error
+    @check_read_only
+    def mutate(self, graphene_info, **kwargs):
+        return resume_partition_backfill(graphene_info, kwargs["backfillId"])
+
+
 @capture_error
 def create_execution_params_and_launch_pipeline_reexec(graphene_info, execution_params_dict):
     # refactored into a helper function here in order to wrap with @capture_error,
@@ -248,6 +275,8 @@ class GrapheneLaunchPipelineReexecutionMutation(graphene.Mutation):
         description = "Re-launch a pipeline run via the run launcher configured on the instance"
         name = "LaunchPipelineReexecutionMutation"
 
+    @capture_error
+    @check_read_only
     def mutate(self, graphene_info, **kwargs):
         return create_execution_params_and_launch_pipeline_reexec(
             graphene_info,
@@ -278,6 +307,8 @@ class GrapheneTerminatePipelineExecutionMutation(graphene.Mutation):
     class Meta:
         name = "TerminatePipelineExecutionMutation"
 
+    @capture_error
+    @check_read_only
     def mutate(self, graphene_info, **kwargs):
         return terminate_pipeline_execution(
             graphene_info,
@@ -289,10 +320,11 @@ class GrapheneTerminatePipelineExecutionMutation(graphene.Mutation):
 class GrapheneReloadRepositoryLocationMutationResult(graphene.Union):
     class Meta:
         types = (
-            GrapheneRepositoryLocation,
+            GrapheneWorkspaceLocationEntry,
             GrapheneReloadNotSupported,
             GrapheneRepositoryLocationNotFound,
-            GrapheneRepositoryLocationLoadFailure,
+            GrapheneReadOnlyError,
+            GraphenePythonError,
         )
         name = "ReloadRepositoryLocationMutationResult"
 
@@ -306,6 +338,8 @@ class GrapheneReloadRepositoryLocationMutation(graphene.Mutation):
     class Meta:
         name = "ReloadRepositoryLocationMutation"
 
+    @capture_error
+    @check_read_only
     def mutate(self, graphene_info, **kwargs):
         location_name = kwargs["repositoryLocationName"]
 
@@ -318,24 +352,19 @@ class GrapheneReloadRepositoryLocationMutation(graphene.Mutation):
             return GrapheneReloadNotSupported(location_name)
 
         # The current workspace context is a WorkspaceRequestContext, which contains a reference to the
-        # repository locations that were present in the root WorkspaceProcessContext the start of the
-        # request. Reloading a repository location modifies the WorkspaceWorkspaceProcessContext, rendeirng
+        # repository locations that were present in the root IWorkspaceProcessContext the start of the
+        # request. Reloading a repository location modifies the IWorkspaceProcessContext, rendeirng
         # our current WorkspaceRequestContext outdated. Therefore, `reload_repository_location` returns
         # an updated WorkspaceRequestContext for us to use.
         new_context = graphene_info.context.reload_repository_location(location_name)
-
-        if new_context.has_repository_location(location_name):
-            return GrapheneRepositoryLocation(new_context.get_repository_location(location_name))
-        else:
-            return GrapheneRepositoryLocationLoadFailure(
-                location_name, new_context.get_repository_location_error(location_name)
-            )
+        return GrapheneWorkspaceLocationEntry(new_context.workspace_snapshot[location_name])
 
 
 class GrapheneReloadWorkspaceMutationResult(graphene.Union):
     class Meta:
         types = (
-            GrapheneRepositoryLocationConnection,
+            GrapheneWorkspace,
+            GrapheneReadOnlyError,
             GraphenePythonError,
         )
         name = "ReloadWorkspaceMutationResult"
@@ -347,13 +376,15 @@ class GrapheneReloadWorkspaceMutation(graphene.Mutation):
     class Meta:
         name = "ReloadWorkspaceMutation"
 
+    @capture_error
+    @check_read_only
     def mutate(self, graphene_info, **_kwargs):
         new_context = graphene_info.context.reload_workspace()
-        return fetch_repository_locations(new_context)
+        return fetch_workspace(new_context)
 
 
 class GrapheneAssetWipeSuccess(graphene.ObjectType):
-    assetKey = graphene.NonNull(GrapheneAssetKey)
+    assetKeys = non_null_list(GrapheneAssetKey)
 
     class Meta:
         name = "AssetWipeSuccess"
@@ -363,6 +394,7 @@ class GrapheneAssetWipeMutationResult(graphene.Union):
     class Meta:
         types = (
             GrapheneAssetNotFoundError,
+            GrapheneReadOnlyError,
             GraphenePythonError,
             GrapheneAssetWipeSuccess,
         )
@@ -373,13 +405,18 @@ class GrapheneAssetWipeMutation(graphene.Mutation):
     Output = graphene.NonNull(GrapheneAssetWipeMutationResult)
 
     class Arguments:
-        assetKey = graphene.Argument(graphene.NonNull(GrapheneAssetKeyInput))
+        assetKeys = graphene.Argument(non_null_list(GrapheneAssetKeyInput))
 
     class Meta:
         name = "AssetWipeMutation"
 
+    @capture_error
+    @check_read_only
     def mutate(self, graphene_info, **kwargs):
-        return wipe_asset(graphene_info, AssetKey.from_graphql_input(kwargs["assetKey"]))
+        return wipe_assets(
+            graphene_info,
+            [AssetKey.from_graphql_input(asset_key) for asset_key in kwargs["assetKeys"]],
+        )
 
 
 class GrapheneMutation(graphene.ObjectType):
@@ -394,8 +431,9 @@ class GrapheneMutation(graphene.ObjectType):
     delete_pipeline_run = GrapheneDeleteRunMutation.Field()
     reload_repository_location = GrapheneReloadRepositoryLocationMutation.Field()
     reload_workspace = GrapheneReloadWorkspaceMutation.Field()
-    wipe_asset = GrapheneAssetWipeMutation.Field()
+    wipe_assets = GrapheneAssetWipeMutation.Field()
     launch_partition_backfill = GrapheneLaunchBackfillMutation.Field()
+    resume_partition_backfill = GrapheneResumeBackfillMutation.Field()
     cancel_partition_backfill = GrapheneCancelBackfillMutation.Field()
 
     class Meta:
