@@ -1,246 +1,28 @@
-# NOTE: pylint isn't smart enough to figure out what's going on with dauphin
-# pylint: disable=unexpected-keyword-arg, no-value-for-parameter
-
+import graphene
 from dagster import check
 from dagster.core.definitions import SolidHandle
 from dagster.core.host_representation import RepresentedPipeline
 from dagster.core.snap import CompositeSolidDefSnap, DependencyStructureIndex, SolidDefSnap
-from dagster_graphql import dauphin
 
-from .config_types import DauphinConfigTypeField
-from .dagster_types import to_dauphin_dagster_type
-
-
-class DauphinSolidContainer(dauphin.Interface):
-    class Meta(object):
-        name = "SolidContainer"
-
-    solids = dauphin.non_null_list("Solid")
-
-
-class DauphinISolidDefinition(dauphin.Interface):
-    class Meta(object):
-        name = "ISolidDefinition"
-
-    name = dauphin.NonNull(dauphin.String)
-    description = dauphin.String()
-    metadata = dauphin.non_null_list("MetadataItemDefinition")
-    input_definitions = dauphin.non_null_list("InputDefinition")
-    output_definitions = dauphin.non_null_list("OutputDefinition")
-    required_resources = dauphin.non_null_list("ResourceRequirement")
-
-
-class IDauphinSolidDefinitionMixin(object):
-    def __init__(self, represented_pipeline, solid_def_name):
-        self._represented_pipeline = check.inst_param(
-            represented_pipeline, "represented_pipeline", RepresentedPipeline
-        )
-        check.str_param(solid_def_name, "solid_def_name")
-        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
-
-    def resolve_metadata(self, graphene_info):
-        return [
-            graphene_info.schema.type_named("MetadataItemDefinition")(key=item[0], value=item[1])
-            for item in self._solid_def_snap.tags.items()
-        ]
-
-    @property
-    def solid_def_name(self):
-        return self._solid_def_snap.name
-
-    def resolve_input_definitions(self, _):
-        return [
-            DauphinInputDefinition(
-                self._represented_pipeline, self.solid_def_name, input_def_snap.name
-            )
-            for input_def_snap in self._solid_def_snap.input_def_snaps
-        ]
-
-    def resolve_output_definitions(self, _):
-        return [
-            DauphinOutputDefinition(
-                self._represented_pipeline, self.solid_def_name, output_def_snap.name
-            )
-            for output_def_snap in self._solid_def_snap.output_def_snaps
-        ]
-
-    def resolve_required_resources(self, graphene_info):
-        return [
-            graphene_info.schema.type_named("ResourceRequirement")(key)
-            for key in self._solid_def_snap.required_resource_keys
-        ]
-
-
-def build_dauphin_solid_definition(represented_pipeline, solid_def_name):
-    check.inst_param(represented_pipeline, "represented_pipeline", RepresentedPipeline)
-    check.str_param(solid_def_name, "solid_def_name")
-
-    solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
-
-    if isinstance(solid_def_snap, SolidDefSnap):
-        return DauphinSolidDefinition(represented_pipeline, solid_def_snap.name)
-
-    if isinstance(solid_def_snap, CompositeSolidDefSnap):
-        return DauphinCompositeSolidDefinition(represented_pipeline, solid_def_snap.name)
-
-    check.failed("Unknown solid definition type {type}".format(type=type(solid_def_snap)))
+from .config_types import GrapheneConfigTypeField
+from .dagster_types import GrapheneDagsterType, to_dagster_type
+from .metadata import GrapheneMetadataItemDefinition
+from .util import non_null_list
 
 
 class _ArgNotPresentSentinel:
     pass
 
 
-class DauphinSolid(dauphin.ObjectType):
-    class Meta(object):
-        name = "Solid"
+class GrapheneInputDefinition(graphene.ObjectType):
+    solid_definition = graphene.NonNull(lambda: GrapheneSolidDefinition)
+    name = graphene.NonNull(graphene.String)
+    description = graphene.String()
+    type = graphene.NonNull(GrapheneDagsterType)
 
-    name = dauphin.NonNull(dauphin.String)
-    definition = dauphin.NonNull("ISolidDefinition")
-    inputs = dauphin.non_null_list("Input")
-    outputs = dauphin.non_null_list("Output")
-
-    def __init__(self, represented_pipeline, solid_name, current_dep_structure):
-        self._represented_pipeline = check.inst_param(
-            represented_pipeline, "represented_pipeline", RepresentedPipeline
-        )
-
-        check.str_param(solid_name, "solid_name")
-        self._current_dep_structure = check.inst_param(
-            current_dep_structure, "current_dep_structure", DependencyStructureIndex
-        )
-        self._solid_invocation_snap = current_dep_structure.get_invocation(solid_name)
-        self._solid_def_snap = represented_pipeline.get_solid_def_snap(
-            self._solid_invocation_snap.solid_def_name
-        )
-        super(DauphinSolid, self).__init__(name=solid_name)
-
-    def get_solid_definition_name(self):
-        return self._solid_def_snap.name
-
-    def get_dauphin_solid_definition(self):
-        return build_dauphin_solid_definition(self._represented_pipeline, self._solid_def_snap.name)
-
-    def resolve_definition(self, _):
-        return self.get_dauphin_solid_definition()
-
-    def resolve_inputs(self, _):
-        return [
-            DauphinInput(
-                self._represented_pipeline,
-                self._current_dep_structure,
-                self._solid_invocation_snap.solid_name,
-                input_def_snap.name,
-            )
-            for input_def_snap in self._solid_def_snap.input_def_snaps
-        ]
-
-    def resolve_outputs(self, _):
-        return [
-            DauphinOutput(
-                self._represented_pipeline,
-                self._current_dep_structure,
-                self._solid_invocation_snap.solid_name,
-                output_def_snap.name,
-            )
-            for output_def_snap in self._solid_def_snap.output_def_snaps
-        ]
-
-
-class DauphinSolidDefinition(dauphin.ObjectType, IDauphinSolidDefinitionMixin):
-    class Meta(object):
-        name = "SolidDefinition"
-        interfaces = [DauphinISolidDefinition]
-
-    config_field = dauphin.Field("ConfigTypeField")
-
-    def __init__(self, represented_pipeline, solid_def_name):
-        check.inst_param(represented_pipeline, "represented_pipeline", RepresentedPipeline)
-        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
-        super(DauphinSolidDefinition, self).__init__(
-            name=solid_def_name, description=self._solid_def_snap.description
-        )
-        IDauphinSolidDefinitionMixin.__init__(self, represented_pipeline, solid_def_name)
-
-    def resolve_config_field(self, _):
-        return (
-            DauphinConfigTypeField(
-                config_schema_snapshot=self._represented_pipeline.config_schema_snapshot,
-                field_snap=self._solid_def_snap.config_field_snap,
-            )
-            if self._solid_def_snap.config_field_snap
-            else None
-        )
-
-
-class DauphinCompositeSolidDefinition(dauphin.ObjectType, IDauphinSolidDefinitionMixin):
-    class Meta(object):
-        name = "CompositeSolidDefinition"
-        interfaces = [DauphinISolidDefinition, DauphinSolidContainer]
-
-    solids = dauphin.non_null_list("Solid")
-    input_mappings = dauphin.non_null_list("InputMapping")
-    output_mappings = dauphin.non_null_list("OutputMapping")
-
-    def __init__(self, represented_pipeline, solid_def_name):
-        self._represented_pipeline = check.inst_param(
-            represented_pipeline, "represented_pipeline", RepresentedPipeline
-        )
-        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
-        self._comp_solid_dep_index = represented_pipeline.get_dep_structure_index(solid_def_name)
-        super(DauphinCompositeSolidDefinition, self).__init__(
-            name=solid_def_name, description=self._solid_def_snap.description
-        )
-        IDauphinSolidDefinitionMixin.__init__(self, represented_pipeline, solid_def_name)
-
-    def resolve_solids(self, _graphene_info):
-        return build_dauphin_solids(self._represented_pipeline, self._comp_solid_dep_index)
-
-    def resolve_output_mappings(self, _):
-        return [
-            DauphinOutputMapping(
-                self._represented_pipeline,
-                self._comp_solid_dep_index,
-                self._solid_def_snap.name,
-                output_def_snap.name,
-            )
-            for output_def_snap in self._solid_def_snap.output_def_snaps
-        ]
-
-    def resolve_input_mappings(self, _):
-        return [
-            DauphinInputMapping(
-                self._represented_pipeline,
-                self._comp_solid_dep_index,
-                self._solid_def_snap.name,
-                input_def_snap.name,
-            )
-            for input_def_snap in self._solid_def_snap.input_def_snaps
-        ]
-
-
-class DauphinSolidHandle(dauphin.ObjectType):
-    class Meta(object):
-        name = "SolidHandle"
-
-    handleID = dauphin.NonNull(dauphin.String)
-    solid = dauphin.NonNull("Solid")
-    parent = dauphin.Field("SolidHandle")
-
-    def __init__(self, handle, solid, parent):
-        self.handleID = check.inst_param(handle, "handle", SolidHandle)
-        self.solid = check.inst_param(solid, "solid", DauphinSolid)
-        self.parent = check.opt_inst_param(parent, "parent", DauphinSolidHandle)
-
-
-class DauphinInputDefinition(dauphin.ObjectType):
-    class Meta(object):
+    class Meta:
         name = "InputDefinition"
 
-    solid_definition = dauphin.NonNull("SolidDefinition")
-    name = dauphin.NonNull(dauphin.String)
-    description = dauphin.String()
-    type = dauphin.NonNull("DagsterType")
-
     def __init__(self, represented_pipeline, solid_def_name, input_def_name):
         self._represented_pipeline = check.inst_param(
             represented_pipeline, "represented_pipeline", RepresentedPipeline
@@ -249,31 +31,33 @@ class DauphinInputDefinition(dauphin.ObjectType):
         check.str_param(input_def_name, "input_def_name")
         solid_def_snap = self._represented_pipeline.get_solid_def_snap(solid_def_name)
         self._input_def_snap = solid_def_snap.get_input_snap(input_def_name)
-        super(DauphinInputDefinition, self).__init__(
-            name=self._input_def_snap.name, description=self._input_def_snap.description,
+        super().__init__(
+            name=self._input_def_snap.name,
+            description=self._input_def_snap.description,
         )
 
     def resolve_type(self, _graphene_info):
-        return to_dauphin_dagster_type(
+        return to_dagster_type(
             self._represented_pipeline.pipeline_snapshot, self._input_def_snap.dagster_type_key
         )
 
-    def resolve_solid_definition(self, _):
-        return build_dauphin_solid_definition(
+    def resolve_solid_definition(self, _graphene_info):
+        return build_solid_definition(
             self._represented_pipeline, self._solid_def_snap.name  # pylint: disable=no-member
         )
 
 
-class DauphinOutputDefinition(dauphin.ObjectType):
-    class Meta(object):
+class GrapheneOutputDefinition(graphene.ObjectType):
+    solid_definition = graphene.NonNull(lambda: GrapheneSolidDefinition)
+    name = graphene.NonNull(graphene.String)
+    description = graphene.String()
+    is_dynamic = graphene.Boolean()
+    type = graphene.NonNull(GrapheneDagsterType)
+
+    class Meta:
         name = "OutputDefinition"
 
-    solid_definition = dauphin.NonNull("SolidDefinition")
-    name = dauphin.NonNull(dauphin.String)
-    description = dauphin.String()
-    type = dauphin.NonNull("DagsterType")
-
-    def __init__(self, represented_pipeline, solid_def_name, output_def_name):
+    def __init__(self, represented_pipeline, solid_def_name, output_def_name, is_dynamic):
         self._represented_pipeline = check.inst_param(
             represented_pipeline, "represented_pipeline", RepresentedPipeline
         )
@@ -283,27 +67,31 @@ class DauphinOutputDefinition(dauphin.ObjectType):
         self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
         self._output_def_snap = self._solid_def_snap.get_output_snap(output_def_name)
 
-        super(DauphinOutputDefinition, self).__init__(
-            name=self._output_def_snap.name, description=self._output_def_snap.description,
+        super().__init__(
+            name=self._output_def_snap.name,
+            description=self._output_def_snap.description,
+            is_dynamic=is_dynamic,
         )
 
-    def resolve_type(self, _):
-        return to_dauphin_dagster_type(
-            self._represented_pipeline.pipeline_snapshot, self._output_def_snap.dagster_type_key,
+    def resolve_type(self, _graphene_info):
+        return to_dagster_type(
+            self._represented_pipeline.pipeline_snapshot,
+            self._output_def_snap.dagster_type_key,
         )
 
-    def resolve_solid_definition(self, _):
-        return build_dauphin_solid_definition(self._represented_pipeline, self._solid_def_snap.name)
+    def resolve_solid_definition(self, _graphene_info):
+        return build_solid_definition(self._represented_pipeline, self._solid_def_snap.name)
 
 
-class DauphinInput(dauphin.ObjectType):
-    class Meta(object):
+class GrapheneInput(graphene.ObjectType):
+    solid = graphene.NonNull(lambda: GrapheneSolid)
+    definition = graphene.NonNull(GrapheneInputDefinition)
+    depends_on = non_null_list(lambda: GrapheneOutput)
+    is_dynamic_collect = graphene.NonNull(graphene.Boolean)
+
+    class Meta:
         name = "Input"
 
-    solid = dauphin.NonNull("Solid")
-    definition = dauphin.NonNull("InputDefinition")
-    depends_on = dauphin.non_null_list("Output")
-
     def __init__(self, represented_pipeline, current_dep_structure, solid_name, input_name):
         self._represented_pipeline = check.inst_param(
             represented_pipeline, "represented_pipeline", RepresentedPipeline
@@ -319,21 +107,23 @@ class DauphinInput(dauphin.ObjectType):
         )
         self._input_def_snap = self._solid_def_snap.get_input_snap(input_name)
 
-        super(DauphinInput, self).__init__()
+        super().__init__()
 
-    def resolve_solid(self, _):
-        return DauphinSolid(
+    def resolve_solid(self, _graphene_info):
+        return GrapheneSolid(
             self._represented_pipeline, self._solid_name, self._current_dep_structure
         )
 
-    def resolve_definition(self, _):
-        return DauphinInputDefinition(
-            self._represented_pipeline, self._solid_def_snap.name, self._input_def_snap.name,
+    def resolve_definition(self, _graphene_info):
+        return GrapheneInputDefinition(
+            self._represented_pipeline,
+            self._solid_def_snap.name,
+            self._input_def_snap.name,
         )
 
-    def resolve_depends_on(self, _):
+    def resolve_depends_on(self, _graphene_info):
         return [
-            DauphinOutput(
+            GrapheneOutput(
                 self._represented_pipeline,
                 self._current_dep_structure,
                 output_handle_snap.solid_name,
@@ -344,15 +134,18 @@ class DauphinInput(dauphin.ObjectType):
             )
         ]
 
+    def resolve_is_dynamic_collect(self, _graphene_info):
+        return self._solid_invocation_snap.input_dep_snap(self._input_name).is_dynamic_collect
 
-class DauphinOutput(dauphin.ObjectType):
-    class Meta(object):
+
+class GrapheneOutput(graphene.ObjectType):
+    solid = graphene.NonNull(lambda: GrapheneSolid)
+    definition = graphene.NonNull(GrapheneOutputDefinition)
+    depended_by = non_null_list(GrapheneInput)
+
+    class Meta:
         name = "Output"
 
-    solid = dauphin.NonNull("Solid")
-    definition = dauphin.NonNull("OutputDefinition")
-    depended_by = dauphin.non_null_list("Input")
-
     def __init__(self, represented_pipeline, current_dep_structure, solid_name, output_name):
         self._represented_pipeline = check.inst_param(
             represented_pipeline, "represented_pipeline", RepresentedPipeline
@@ -367,21 +160,24 @@ class DauphinOutput(dauphin.ObjectType):
             self._solid_invocation_snap.solid_def_name
         )
         self._output_def_snap = self._solid_def_snap.get_output_snap(output_name)
-        super(DauphinOutput, self).__init__()
+        super().__init__()
 
     def resolve_solid(self, _):
-        return DauphinSolid(
+        return GrapheneSolid(
             self._represented_pipeline, self._solid_name, self._current_dep_structure
         )
 
-    def resolve_definition(self, _):
-        return DauphinOutputDefinition(
-            self._represented_pipeline, self._solid_def_snap.name, self._output_name,
+    def resolve_definition(self, _graphene_info):
+        return GrapheneOutputDefinition(
+            self._represented_pipeline,
+            self._solid_def_snap.name,
+            self._output_name,
+            self._output_def_snap.is_dynamic,
         )
 
-    def resolve_depended_by(self, _):
+    def resolve_depended_by(self, _graphene_info):
         return [
-            DauphinInput(
+            GrapheneInput(
                 self._represented_pipeline,
                 self._current_dep_structure,
                 input_handle_snap.solid_name,
@@ -393,15 +189,19 @@ class DauphinOutput(dauphin.ObjectType):
         ]
 
 
-class DauphinInputMapping(dauphin.ObjectType):
-    class Meta(object):
+class GrapheneInputMapping(graphene.ObjectType):
+    mapped_input = graphene.NonNull(GrapheneInput)
+    definition = graphene.NonNull(GrapheneInputDefinition)
+
+    class Meta:
         name = "InputMapping"
 
-    mapped_input = dauphin.NonNull("Input")
-    definition = dauphin.NonNull("InputDefinition")
-
     def __init__(
-        self, represented_pipeline, current_dep_index, solid_def_name, input_name,
+        self,
+        represented_pipeline,
+        current_dep_index,
+        solid_def_name,
+        input_name,
     ):
         self._represented_pipeline = check.inst_param(
             represented_pipeline, "represented_pipeline", RepresentedPipeline
@@ -411,33 +211,37 @@ class DauphinInputMapping(dauphin.ObjectType):
         )
         self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
         self._input_mapping_snap = self._solid_def_snap.get_input_mapping_snap(input_name)
-        super(DauphinInputMapping, self).__init__()
+        super().__init__()
 
-    def resolve_mapped_input(self, _):
-        return DauphinInput(
+    def resolve_mapped_input(self, _graphene_info):
+        return GrapheneInput(
             self._represented_pipeline,
             self._current_dep_index,
             self._input_mapping_snap.mapped_solid_name,
             self._input_mapping_snap.mapped_input_name,
         )
 
-    def resolve_definition(self, _):
-        return DauphinInputDefinition(
+    def resolve_definition(self, _graphene_info):
+        return GrapheneInputDefinition(
             self._represented_pipeline,
             self._solid_def_snap.name,
             self._input_mapping_snap.external_input_name,
         )
 
 
-class DauphinOutputMapping(dauphin.ObjectType):
-    class Meta(object):
+class GrapheneOutputMapping(graphene.ObjectType):
+    mapped_output = graphene.NonNull(GrapheneOutput)
+    definition = graphene.NonNull(GrapheneOutputDefinition)
+
+    class Meta:
         name = "OutputMapping"
 
-    mapped_output = dauphin.NonNull("Output")
-    definition = dauphin.NonNull("OutputDefinition")
-
     def __init__(
-        self, represented_pipeline, current_dep_index, solid_def_name, output_name,
+        self,
+        represented_pipeline,
+        current_dep_index,
+        solid_def_name,
+        output_name,
     ):
         self._represented_pipeline = check.inst_param(
             represented_pipeline, "represented_pipeline", RepresentedPipeline
@@ -447,77 +251,63 @@ class DauphinOutputMapping(dauphin.ObjectType):
         )
         self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
         self._output_mapping_snap = self._solid_def_snap.get_output_mapping_snap(output_name)
-        super(DauphinOutputMapping, self).__init__()
+        self._output_def_snap = self._solid_def_snap.get_output_snap(output_name)
 
-    def resolve_mapped_output(self, _):
-        return DauphinOutput(
+        super().__init__()
+
+    def resolve_mapped_output(self, _graphene_info):
+        return GrapheneOutput(
             self._represented_pipeline,
             self._current_dep_index,
             self._output_mapping_snap.mapped_solid_name,
             self._output_mapping_snap.mapped_output_name,
         )
 
-    def resolve_definition(self, _):
-        return DauphinOutputDefinition(
+    def resolve_definition(self, _graphene_info):
+        return GrapheneOutputDefinition(
             self._represented_pipeline,
             self._solid_def_snap.name,
             self._output_mapping_snap.external_output_name,
+            self._output_def_snap.is_dynamic,
         )
 
 
-class DauphinResourceRequirement(dauphin.ObjectType):
-    class Meta(object):
+class GrapheneResourceRequirement(graphene.ObjectType):
+    resource_key = graphene.NonNull(graphene.String)
+
+    class Meta:
         name = "ResourceRequirement"
 
-    resource_key = dauphin.NonNull(dauphin.String)
-
     def __init__(self, resource_key):
+        super().__init__()
         self.resource_key = resource_key
 
 
-class DauphinUsedSolid(dauphin.ObjectType):
-    class Meta(object):
-        name = "UsedSolid"
-        description = """A solid definition and it's invocations within the repo."""
-
-    definition = dauphin.NonNull("ISolidDefinition")
-    invocations = dauphin.non_null_list("SolidInvocationSite")
-
-
-class DauphinSolidInvocationSite(dauphin.ObjectType):
-    class Meta(object):
-        name = "SolidInvocationSite"
-        description = """An invocation of a solid within a repo."""
-
-    pipeline = dauphin.NonNull("Pipeline")
-    solidHandle = dauphin.NonNull("SolidHandle")
-
-
-def build_dauphin_solids(represented_pipeline, current_dep_index):
+def build_solids(represented_pipeline, current_dep_index):
     check.inst_param(represented_pipeline, "represented_pipeline", RepresentedPipeline)
     return sorted(
         [
-            DauphinSolid(represented_pipeline, solid_name, current_dep_index)
+            GrapheneSolid(represented_pipeline, solid_name, current_dep_index)
             for solid_name in current_dep_index.solid_invocation_names
         ],
         key=lambda solid: solid.name,
     )
 
 
-def build_dauphin_solid_handles(represented_pipeline, current_dep_index, parent=None):
+def build_solid_handles(represented_pipeline, current_dep_index, parent=None):
     check.inst_param(represented_pipeline, "represented_pipeline", RepresentedPipeline)
-    check.opt_inst_param(parent, "parent", DauphinSolidHandle)
+    check.opt_inst_param(parent, "parent", GrapheneSolidHandle)
     all_handle = []
     for solid_invocation in current_dep_index.solid_invocations:
         solid_name, solid_def_name = solid_invocation.solid_name, solid_invocation.solid_def_name
-        handle = DauphinSolidHandle(
-            solid=DauphinSolid(represented_pipeline, solid_name, current_dep_index),
+        handle = GrapheneSolidHandle(
+            solid=GrapheneSolid(represented_pipeline, solid_name, current_dep_index),
             handle=SolidHandle(solid_name, parent.handleID if parent else None),
             parent=parent if parent else None,
         )
         solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
         if isinstance(solid_def_snap, CompositeSolidDefSnap):
-            all_handle += build_dauphin_solid_handles(
+            all_handle += build_solid_handles(
                 represented_pipeline,
                 represented_pipeline.get_dep_structure_index(solid_def_name),
                 handle,
@@ -526,3 +316,241 @@ def build_dauphin_solid_handles(represented_pipeline, current_dep_index, parent=
         all_handle.append(handle)
 
     return all_handle
+
+
+class GrapheneISolidDefinition(graphene.Interface):
+    name = graphene.NonNull(graphene.String)
+    description = graphene.String()
+    metadata = non_null_list(GrapheneMetadataItemDefinition)
+    input_definitions = non_null_list(GrapheneInputDefinition)
+    output_definitions = non_null_list(GrapheneOutputDefinition)
+
+    class Meta:
+        name = "ISolidDefinition"
+
+
+class ISolidDefinitionMixin:
+    def __init__(self, represented_pipeline, solid_def_name):
+        self._represented_pipeline = check.inst_param(
+            represented_pipeline, "represented_pipeline", RepresentedPipeline
+        )
+        check.str_param(solid_def_name, "solid_def_name")
+        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
+
+    def resolve_metadata(self, _graphene_info):
+        return [
+            GrapheneMetadataItemDefinition(key=item[0], value=item[1])
+            for item in self._solid_def_snap.tags.items()
+        ]
+
+    @property
+    def solid_def_name(self):
+        return self._solid_def_snap.name
+
+    def resolve_input_definitions(self, _graphene_info):
+        return [
+            GrapheneInputDefinition(
+                self._represented_pipeline, self.solid_def_name, input_def_snap.name
+            )
+            for input_def_snap in self._solid_def_snap.input_def_snaps
+        ]
+
+    def resolve_output_definitions(self, _graphene_info):
+        return [
+            GrapheneOutputDefinition(
+                self._represented_pipeline,
+                self.solid_def_name,
+                output_def_snap.name,
+                output_def_snap.is_dynamic,
+            )
+            for output_def_snap in self._solid_def_snap.output_def_snaps
+        ]
+
+
+class GrapheneSolidDefinition(graphene.ObjectType, ISolidDefinitionMixin):
+    config_field = graphene.Field(GrapheneConfigTypeField)
+    required_resources = non_null_list(GrapheneResourceRequirement)
+
+    class Meta:
+        interfaces = (GrapheneISolidDefinition,)
+        name = "SolidDefinition"
+
+    def __init__(self, represented_pipeline, solid_def_name):
+        check.inst_param(represented_pipeline, "represented_pipeline", RepresentedPipeline)
+        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
+        super().__init__(name=solid_def_name, description=self._solid_def_snap.description)
+        ISolidDefinitionMixin.__init__(self, represented_pipeline, solid_def_name)
+
+    def resolve_config_field(self, _graphene_info):
+        return (
+            GrapheneConfigTypeField(
+                config_schema_snapshot=self._represented_pipeline.config_schema_snapshot,
+                field_snap=self._solid_def_snap.config_field_snap,
+            )
+            if self._solid_def_snap.config_field_snap
+            else None
+        )
+
+    def resolve_required_resources(self, _graphene_info):
+        return [
+            GrapheneResourceRequirement(key) for key in self._solid_def_snap.required_resource_keys
+        ]
+
+
+class GrapheneSolid(graphene.ObjectType):
+    name = graphene.NonNull(graphene.String)
+    definition = graphene.NonNull(lambda: GrapheneISolidDefinition)
+    inputs = non_null_list(GrapheneInput)
+    outputs = non_null_list(GrapheneOutput)
+    is_dynamic_mapped = graphene.NonNull(graphene.Boolean)
+
+    class Meta:
+        name = "Solid"
+
+    def __init__(self, represented_pipeline, solid_name, current_dep_structure):
+        self._represented_pipeline = check.inst_param(
+            represented_pipeline, "represented_pipeline", RepresentedPipeline
+        )
+
+        check.str_param(solid_name, "solid_name")
+        self._current_dep_structure = check.inst_param(
+            current_dep_structure, "current_dep_structure", DependencyStructureIndex
+        )
+        self._solid_invocation_snap = current_dep_structure.get_invocation(solid_name)
+        self._solid_def_snap = represented_pipeline.get_solid_def_snap(
+            self._solid_invocation_snap.solid_def_name
+        )
+        super().__init__(name=solid_name)
+
+    def get_solid_definition_name(self):
+        return self._solid_def_snap.name
+
+    def get_solid_definition(self):
+        return build_solid_definition(self._represented_pipeline, self._solid_def_snap.name)
+
+    def resolve_definition(self, _graphene_info):
+        return self.get_solid_definition()
+
+    def resolve_inputs(self, _graphene_info):
+        return [
+            GrapheneInput(
+                self._represented_pipeline,
+                self._current_dep_structure,
+                self._solid_invocation_snap.solid_name,
+                input_def_snap.name,
+            )
+            for input_def_snap in self._solid_def_snap.input_def_snaps
+        ]
+
+    def resolve_outputs(self, _graphene_info):
+        return [
+            GrapheneOutput(
+                self._represented_pipeline,
+                self._current_dep_structure,
+                self._solid_invocation_snap.solid_name,
+                output_def_snap.name,
+            )
+            for output_def_snap in self._solid_def_snap.output_def_snaps
+        ]
+
+    def resolve_is_dynamic_mapped(self, _graphene_info):
+        return self._solid_invocation_snap.is_dynamic_mapped
+
+
+class GrapheneSolidContainer(graphene.Interface):
+    solids = non_null_list(GrapheneSolid)
+
+    class Meta:
+        name = "SolidContainer"
+
+
+class GrapheneCompositeSolidDefinition(graphene.ObjectType, ISolidDefinitionMixin):
+    solids = non_null_list(GrapheneSolid)
+    input_mappings = non_null_list(GrapheneInputMapping)
+    output_mappings = non_null_list(GrapheneOutputMapping)
+
+    class Meta:
+        interfaces = (GrapheneISolidDefinition, GrapheneSolidContainer)
+        name = "CompositeSolidDefinition"
+
+    def __init__(self, represented_pipeline, solid_def_name):
+        self._represented_pipeline = check.inst_param(
+            represented_pipeline, "represented_pipeline", RepresentedPipeline
+        )
+        self._solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
+        self._comp_solid_dep_index = represented_pipeline.get_dep_structure_index(solid_def_name)
+        super().__init__(name=solid_def_name, description=self._solid_def_snap.description)
+        ISolidDefinitionMixin.__init__(self, represented_pipeline, solid_def_name)
+
+    def resolve_solids(self, _graphene_info):
+        return build_solids(self._represented_pipeline, self._comp_solid_dep_index)
+
+    def resolve_output_mappings(self, _graphene_info):
+        return [
+            GrapheneOutputMapping(
+                self._represented_pipeline,
+                self._comp_solid_dep_index,
+                self._solid_def_snap.name,
+                output_def_snap.name,
+            )
+            for output_def_snap in self._solid_def_snap.output_def_snaps
+        ]
+
+    def resolve_input_mappings(self, _graphene_info):
+        return [
+            GrapheneInputMapping(
+                self._represented_pipeline,
+                self._comp_solid_dep_index,
+                self._solid_def_snap.name,
+                input_def_snap.name,
+            )
+            for input_def_snap in self._solid_def_snap.input_def_snaps
+        ]
+
+
+class GrapheneSolidHandle(graphene.ObjectType):
+    handleID = graphene.NonNull(graphene.String)
+    solid = graphene.NonNull(GrapheneSolid)
+    parent = graphene.Field(lambda: GrapheneSolidHandle)
+
+    class Meta:
+        name = "SolidHandle"
+
+    def __init__(self, handle, solid, parent):
+        super().__init__()
+        # FIXME this really seems wrong
+        self.handleID = check.inst_param(handle, "handle", SolidHandle)
+        self.solid = check.inst_param(solid, "solid", GrapheneSolid)
+        self.parent = check.opt_inst_param(parent, "parent", GrapheneSolidHandle)
+
+
+def build_solid_definition(represented_pipeline, solid_def_name):
+    check.inst_param(represented_pipeline, "represented_pipeline", RepresentedPipeline)
+    check.str_param(solid_def_name, "solid_def_name")
+
+    solid_def_snap = represented_pipeline.get_solid_def_snap(solid_def_name)
+
+    if isinstance(solid_def_snap, SolidDefSnap):
+        return GrapheneSolidDefinition(represented_pipeline, solid_def_snap.name)
+
+    if isinstance(solid_def_snap, CompositeSolidDefSnap):
+        return GrapheneCompositeSolidDefinition(represented_pipeline, solid_def_snap.name)
+
+    check.failed("Unknown solid definition type {type}".format(type=type(solid_def_snap)))
+
+
+types = [
+    GrapheneCompositeSolidDefinition,
+    GrapheneInput,
+    GrapheneInputDefinition,
+    GrapheneInputMapping,
+    GrapheneISolidDefinition,
+    GrapheneOutput,
+    GrapheneOutputDefinition,
+    GrapheneOutputMapping,
+    GrapheneResourceRequirement,
+    GrapheneSolid,
+    GrapheneSolidContainer,
+    GrapheneSolidDefinition,
+    GrapheneSolidHandle,
+]
