@@ -21,6 +21,7 @@ from dagster import (
     Field,
     InputDefinition,
     Int,
+    Materialization,
     ModeDefinition,
     Noneable,
     Nothing,
@@ -31,6 +32,7 @@ from dagster import (
     PresetDefinition,
     PythonObjectDagsterType,
     ScheduleDefinition,
+    ScheduleExecutionContext,
     String,
     check,
     composite_solid,
@@ -50,10 +52,10 @@ from dagster import (
 )
 from dagster.cli.workspace.load import location_origin_from_python_file
 from dagster.core.definitions.decorators.sensor import sensor
-from dagster.core.definitions.partition import last_empty_partition
 from dagster.core.definitions.reconstructable import ReconstructableRepository
 from dagster.core.definitions.sensor import RunRequest, SkipReason
 from dagster.core.log_manager import coerce_valid_log_level
+from dagster.core.storage.pipeline_run import PipelineRunStatus, PipelineRunsFilter
 from dagster.core.storage.tags import RESUME_RETRY_TAG
 from dagster.core.test_utils import today_at_midnight
 from dagster.experimental import DynamicOutput, DynamicOutputDefinition
@@ -66,6 +68,8 @@ from dagster_graphql.test.utils import (
     main_repo_name,
 )
 
+LONG_INT = 29119888133298982934829348
+
 
 @dagster_type_loader(String)
 def df_input_schema(_context, path):
@@ -109,8 +113,8 @@ def get_main_external_repo():
         attribute=main_repo_name(),
         working_directory=None,
         location_name=main_repo_location_name(),
-    ).create_handle() as handle:
-        yield handle.create_location().get_repository(main_repo_name())
+    ).create_test_location() as location:
+        yield location.get_repository(main_repo_name())
 
 
 @lambda_solid(
@@ -263,9 +267,7 @@ def pipeline_with_expectations():
             success=True,
             label="always_true",
             description="Successful",
-            metadata_entries=[
-                EventMetadataEntry.json(label="data", data={"reason": "Just because."})
-            ],
+            metadata={"data": {"reason": "Just because."}},
         )
 
     @solid(output_defs=[])
@@ -274,9 +276,7 @@ def pipeline_with_expectations():
             success=False,
             label="always_false",
             description="Failure",
-            metadata_entries=[
-                EventMetadataEntry.json(label="data", data={"reason": "Relentless pessimism."})
-            ],
+            metadata={"data": {"reason": "Relentless pessimism."}},
         )
 
     @solid(output_defs=[])
@@ -636,6 +636,10 @@ def materialization_pipeline():
                 EventMetadataEntry.python_artifact(file_relative_path, "python function"),
                 EventMetadataEntry.float(1.2, "float"),
                 EventMetadataEntry.int(1, "int"),
+                EventMetadataEntry.float(float("nan"), "float NaN"),
+                EventMetadataEntry.int(LONG_INT, "long int"),
+                EventMetadataEntry.pipeline_run("fake_run_id", "pipeline run"),
+                EventMetadataEntry.asset(AssetKey("my_asset"), "my asset"),
             ],
         )
         yield Output(None)
@@ -885,6 +889,25 @@ def get_retry_multi_execution_params(graphql_context, should_fail, retry_id=None
     }
 
 
+def last_empty_partition(context, partition_set_def):
+    check.inst_param(context, "context", ScheduleExecutionContext)
+    partition_set_def = check.inst_param(
+        partition_set_def, "partition_set_def", PartitionSetDefinition
+    )
+
+    partitions = partition_set_def.get_partitions(context.scheduled_execution_time)
+    if not partitions:
+        return None
+    selected = None
+    for partition in reversed(partitions):
+        filters = PipelineRunsFilter.for_partition(partition_set_def, partition)
+        matching = context.instance.get_runs(filters)
+        if not any(run.status == PipelineRunStatus.SUCCESS for run in matching):
+            selected = partition
+            break
+    return selected
+
+
 def define_schedules():
     integer_partition_set = PartitionSetDefinition(
         name="scheduled_integer_partitions",
@@ -1159,6 +1182,33 @@ def chained_failure_pipeline():
     after_failure(conditionally_fail(always_succeed()))
 
 
+@pipeline
+def backcompat_materialization_pipeline():
+    @solid
+    def backcompat_materialize(_):
+        yield Materialization(
+            asset_key="all_types",
+            description="a materialization with all metadata types",
+            metadata_entries=[
+                EventMetadataEntry.text("text is cool", "text"),
+                EventMetadataEntry.url("https://bigty.pe/neato", "url"),
+                EventMetadataEntry.fspath("/tmp/awesome", "path"),
+                EventMetadataEntry.json({"is_dope": True}, "json"),
+                EventMetadataEntry.python_artifact(EventMetadataEntry, "python class"),
+                EventMetadataEntry.python_artifact(file_relative_path, "python function"),
+                EventMetadataEntry.float(1.2, "float"),
+                EventMetadataEntry.int(1, "int"),
+                EventMetadataEntry.float(float("nan"), "float NaN"),
+                EventMetadataEntry.int(LONG_INT, "long int"),
+                EventMetadataEntry.pipeline_run("fake_run_id", "pipeline run"),
+                EventMetadataEntry.asset(AssetKey("my_asset"), "my asset"),
+            ],
+        )
+        yield Output(None)
+
+    backcompat_materialize()
+
+
 @repository
 def empty_repo():
     return []
@@ -1203,6 +1253,7 @@ def define_pipelines():
         dynamic_pipeline,
         asset_lineage_pipeline,
         partitioned_asset_lineage_pipeline,
+        backcompat_materialization_pipeline,
     ]
 
 
