@@ -3,7 +3,7 @@ from dagster.config.config_type import ALL_CONFIG_BUILTINS
 from dagster.utils import file_relative_path
 from dagster_graphql.test.utils import execute_dagster_graphql, infer_pipeline_selector
 
-from .graphql_context_test_suite import ReadonlyGraphQLContextTestMatrix
+from .graphql_context_test_suite import NonLaunchableGraphQLContextTestMatrix
 from .setup import csv_hello_world_solids_config
 
 CONFIG_VALIDATION_QUERY = """
@@ -101,11 +101,15 @@ def execute_config_graphql(context, pipeline_name, run_config, mode):
     return execute_dagster_graphql(
         context,
         CONFIG_VALIDATION_QUERY,
-        {"runConfigData": run_config, "pipeline": selector, "mode": mode,},
+        {
+            "runConfigData": run_config,
+            "pipeline": selector,
+            "mode": mode,
+        },
     )
 
 
-class TestConfigTypes(ReadonlyGraphQLContextTestMatrix):
+class TestConfigTypes(NonLaunchableGraphQLContextTestMatrix):
     def test_pipeline_not_found(self, graphql_context):
         result = execute_config_graphql(
             graphql_context, pipeline_name="nope", run_config={}, mode="default"
@@ -527,7 +531,9 @@ class TestConfigTypes(ReadonlyGraphQLContextTestMatrix):
     def test_smoke_test_config_type_system(self, graphql_context):
         selector = infer_pipeline_selector(graphql_context, "more_complicated_nested_config")
         result = execute_dagster_graphql(
-            graphql_context, ALL_CONFIG_TYPES_QUERY, {"selector": selector, "mode": "default"},
+            graphql_context,
+            ALL_CONFIG_TYPES_QUERY,
+            {"selector": selector, "mode": "default"},
         )
 
         config_types_data = result.data["runConfigSchemaOrError"]["allConfigTypes"]
