@@ -1,13 +1,21 @@
+import time
 import uuid
 
 from dagster.core.storage.pipeline_run import PipelineRunsFilter
 from dagster.utils import file_relative_path, merge_dicts
 from dagster.utils.test import get_temp_file_name
-from dagster_graphql.client.query import LAUNCH_PIPELINE_EXECUTION_MUTATION, SUBSCRIPTION_QUERY
+from dagster_graphql.client.query import (
+    LAUNCH_PIPELINE_EXECUTION_MUTATION,
+    RUN_EVENTS_QUERY,
+    SUBSCRIPTION_QUERY,
+)
 from dagster_graphql.test.utils import execute_dagster_graphql, infer_pipeline_selector
 from graphql import parse
 
-from .graphql_context_test_suite import ExecutingGraphQLContextTestMatrix
+from .graphql_context_test_suite import (
+    ExecutingGraphQLContextTestMatrix,
+    ReadonlyGraphQLContextTestMatrix,
+)
 from .setup import csv_hello_world_solids_config
 from .utils import sync_execute_get_run_log_data
 
@@ -42,7 +50,12 @@ class TestExecutePipeline(ExecutingGraphQLContextTestMatrix):
         result = execute_dagster_graphql(
             graphql_context,
             LAUNCH_PIPELINE_EXECUTION_MUTATION,
-            variables={"executionParams": {"selector": selector, "preset": "test_inline",}},
+            variables={
+                "executionParams": {
+                    "selector": selector,
+                    "preset": "test_inline",
+                }
+            },
         )
 
         assert not result.errors
@@ -60,7 +73,12 @@ class TestExecutePipeline(ExecutingGraphQLContextTestMatrix):
         result = execute_dagster_graphql(
             graphql_context,
             LAUNCH_PIPELINE_EXECUTION_MUTATION,
-            variables={"executionParams": {"selector": selector, "mode": "default",},},
+            variables={
+                "executionParams": {
+                    "selector": selector,
+                    "mode": "default",
+                },
+            },
         )
 
         assert not result.errors
@@ -76,13 +94,38 @@ class TestExecutePipeline(ExecutingGraphQLContextTestMatrix):
             == "hello_world_with_tags"
         )
 
+        # ensure provided tags override def tags
+        result = execute_dagster_graphql(
+            graphql_context,
+            LAUNCH_PIPELINE_EXECUTION_MUTATION,
+            variables={
+                "executionParams": {
+                    "selector": selector,
+                    "mode": "default",
+                    "executionMetadata": {
+                        "tags": [{"key": "tag_key", "value": "new_tag_value"}],
+                    },
+                },
+            },
+        )
+
+        assert not result.errors
+        assert result.data["launchPipelineExecution"]["run"]["tags"] == [
+            {"key": "tag_key", "value": "new_tag_value"}
+        ]
+
     def test_basic_start_pipeline_execution_with_non_existent_preset(self, graphql_context):
         selector = infer_pipeline_selector(graphql_context, "csv_hello_world")
 
         result = execute_dagster_graphql(
             graphql_context,
             LAUNCH_PIPELINE_EXECUTION_MUTATION,
-            variables={"executionParams": {"selector": selector, "preset": "undefined_preset",}},
+            variables={
+                "executionParams": {
+                    "selector": selector,
+                    "preset": "undefined_preset",
+                }
+            },
         )
 
         assert not result.errors
@@ -101,7 +144,12 @@ class TestExecutePipeline(ExecutingGraphQLContextTestMatrix):
         result = execute_dagster_graphql(
             graphql_context,
             LAUNCH_PIPELINE_EXECUTION_MUTATION,
-            variables={"executionParams": {"selector": subset_selector, "preset": "test_inline",}},
+            variables={
+                "executionParams": {
+                    "selector": subset_selector,
+                    "preset": "test_inline",
+                }
+            },
         )
 
         # while illegally defining selector.solid_selection
@@ -207,6 +255,27 @@ class TestExecutePipeline(ExecutingGraphQLContextTestMatrix):
         assert result.data["launchPipelineExecution"]["__typename"] == "PipelineNotFoundError"
         assert result.data["launchPipelineExecution"]["pipelineName"] == "sjkdfkdjkf"
 
+    def _csv_hello_world_event_sequence(self):
+        # expected non engine event sequence from executing csv_hello_world pipeline
+        return [
+            "PipelineStartingEvent",
+            "PipelineStartEvent",
+            "LogsCapturedEvent",
+            "ExecutionStepStartEvent",
+            "ExecutionStepInputEvent",
+            "ExecutionStepOutputEvent",
+            "HandledOutputEvent",
+            "ExecutionStepSuccessEvent",
+            "LogsCapturedEvent",
+            "ExecutionStepStartEvent",
+            "LoadedInputEvent",
+            "ExecutionStepInputEvent",
+            "ExecutionStepOutputEvent",
+            "HandledOutputEvent",
+            "ExecutionStepSuccessEvent",
+            "PipelineSuccessEvent",
+        ]
+
     def test_basic_start_pipeline_execution_and_subscribe(self, graphql_context):
         selector = infer_pipeline_selector(graphql_context, "csv_hello_world")
         run_logs = sync_execute_get_run_log_data(
@@ -232,45 +301,142 @@ class TestExecutePipeline(ExecutingGraphQLContextTestMatrix):
             for message in run_logs["messages"]
             if message["__typename"] != "EngineEvent"
         ]
-        expected_non_engine_event_types = [
-            "PipelineStartEvent",
-            "ExecutionStepStartEvent",
-            "ExecutionStepInputEvent",
-            "ExecutionStepOutputEvent",
-            "ObjectStoreOperationEvent",
-            "ExecutionStepSuccessEvent",
-            "ExecutionStepStartEvent",
-            "ObjectStoreOperationEvent",
-            "ExecutionStepInputEvent",
-            "ExecutionStepOutputEvent",
-            "ObjectStoreOperationEvent",
-            "ExecutionStepSuccessEvent",
-            "PipelineSuccessEvent",
+
+        assert non_engine_event_types == self._csv_hello_world_event_sequence()
+
+    def test_basic_start_pipeline_and_fetch(self, graphql_context):
+        selector = infer_pipeline_selector(graphql_context, "csv_hello_world")
+        exc_result = execute_dagster_graphql(
+            graphql_context,
+            LAUNCH_PIPELINE_EXECUTION_MUTATION,
+            variables={
+                "executionParams": {
+                    "selector": selector,
+                    "runConfigData": {
+                        "solids": {
+                            "sum_solid": {
+                                "inputs": {"num": file_relative_path(__file__, "../data/num.csv")}
+                            }
+                        }
+                    },
+                    "mode": "default",
+                }
+            },
+        )
+
+        assert not exc_result.errors
+        assert exc_result.data
+        assert (
+            exc_result.data["launchPipelineExecution"]["__typename"] == "LaunchPipelineRunSuccess"
+        )
+
+        # block until run finishes
+        graphql_context.instance.run_launcher.join()
+
+        events_result = execute_dagster_graphql(
+            graphql_context,
+            RUN_EVENTS_QUERY,
+            variables={"runId": exc_result.data["launchPipelineExecution"]["run"]["runId"]},
+        )
+
+        assert not events_result.errors
+        assert events_result.data
+        assert events_result.data["pipelineRunOrError"]["__typename"] == "PipelineRun"
+
+        non_engine_event_types = [
+            message["__typename"]
+            for message in events_result.data["pipelineRunOrError"]["events"]
+            if message["__typename"] != "EngineEvent"
         ]
-        assert non_engine_event_types == expected_non_engine_event_types
+        assert non_engine_event_types == self._csv_hello_world_event_sequence()
+
+    def test_basic_start_pipeline_and_poll(self, graphql_context):
+        selector = infer_pipeline_selector(graphql_context, "csv_hello_world")
+        exc_result = execute_dagster_graphql(
+            graphql_context,
+            LAUNCH_PIPELINE_EXECUTION_MUTATION,
+            variables={
+                "executionParams": {
+                    "selector": selector,
+                    "runConfigData": {
+                        "solids": {
+                            "sum_solid": {
+                                "inputs": {"num": file_relative_path(__file__, "../data/num.csv")}
+                            }
+                        }
+                    },
+                    "mode": "default",
+                }
+            },
+        )
+
+        assert not exc_result.errors
+        assert exc_result.data
+        assert (
+            exc_result.data["launchPipelineExecution"]["__typename"] == "LaunchPipelineRunSuccess"
+        )
+
+        def _fetch_events(after):
+            events_result = execute_dagster_graphql(
+                graphql_context,
+                RUN_EVENTS_QUERY,
+                variables={
+                    "runId": exc_result.data["launchPipelineExecution"]["run"]["runId"],
+                    "after": after,
+                },
+            )
+            assert not events_result.errors
+            assert events_result.data
+            assert events_result.data["pipelineRunOrError"]["__typename"] == "PipelineRun"
+            return events_result.data["pipelineRunOrError"]["events"]
+
+        full_logs = []
+        cursor = -1
+        iters = 0
+
+        # do 3 polls, then fetch after waiting for execution to finish
+        while iters < 3:
+            full_logs.extend(_fetch_events(cursor))
+            cursor = len(full_logs) - 1
+            iters += 1
+            time.sleep(0.05)  # 50ms
+
+        # block until run finishes
+        graphql_context.instance.run_launcher.join()
+        full_logs.extend(_fetch_events(cursor))
+
+        non_engine_event_types = [
+            message["__typename"] for message in full_logs if message["__typename"] != "EngineEvent"
+        ]
+        assert non_engine_event_types == self._csv_hello_world_event_sequence()
 
     def test_subscription_query_error(self, graphql_context):
         selector = infer_pipeline_selector(graphql_context, "naughty_programmer_pipeline")
         run_logs = sync_execute_get_run_log_data(
             context=graphql_context,
-            variables={"executionParams": {"selector": selector, "mode": "default",}},
+            variables={
+                "executionParams": {
+                    "selector": selector,
+                    "mode": "default",
+                }
+            },
         )
 
         assert run_logs["__typename"] == "PipelineRunLogsSubscriptionSuccess"
 
         step_run_log_entry = _get_step_run_log_entry(
-            run_logs, "throw_a_thing.compute", "ExecutionStepFailureEvent"
+            run_logs, "throw_a_thing", "ExecutionStepFailureEvent"
         )
 
         assert step_run_log_entry
         # Confirm that it is the user stack
 
-        assert step_run_log_entry["message"] == 'Execution of step "throw_a_thing.compute" failed.'
+        assert step_run_log_entry["message"] == 'Execution of step "throw_a_thing" failed.'
         assert step_run_log_entry["error"]
         assert step_run_log_entry["level"] == "ERROR"
-        assert isinstance(step_run_log_entry["error"]["stack"], list)
+        assert isinstance(step_run_log_entry["error"]["cause"]["stack"], list)
 
-        assert "bad programmer" in step_run_log_entry["error"]["stack"][-1]
+        assert "bad programmer" in step_run_log_entry["error"]["cause"]["stack"][-1]
 
     def test_subscribe_bad_run_id(self, graphql_context):
         run_id = "nope"
@@ -295,7 +461,11 @@ class TestExecutePipeline(ExecutingGraphQLContextTestMatrix):
         result = sync_execute_get_run_log_data(
             context=graphql_context,
             variables={
-                "executionParams": {"selector": selector, "runConfigData": None, "mode": "default",}
+                "executionParams": {
+                    "selector": selector,
+                    "runConfigData": None,
+                    "mode": "default",
+                }
             },
         )
         logs = result["messages"]
@@ -323,10 +493,12 @@ class TestExecutePipeline(ExecutingGraphQLContextTestMatrix):
         assert has_event_of_type(logs, "PipelineSuccessEvent")
         assert not has_event_of_type(logs, "PipelineFailureEvent")
 
-        assert first_event_of_type(logs, "PipelineStartEvent")["level"] == "DEBUG"
+        start_event = first_event_of_type(logs, "PipelineStartEvent")
+        assert start_event["level"] == "DEBUG"
+        assert start_event["eventType"] == "PIPELINE_START"
 
-        sum_solid_output = get_step_output_event(logs, "sum_solid.compute")
-        assert sum_solid_output["stepKey"] == "sum_solid.compute"
+        sum_solid_output = get_step_output_event(logs, "sum_solid")
+        assert sum_solid_output["stepKey"] == "sum_solid"
 
     def test_basic_filesystem_sync_execution(self, graphql_context):
         selector = infer_pipeline_selector(graphql_context, "csv_hello_world")
@@ -336,7 +508,8 @@ class TestExecutePipeline(ExecutingGraphQLContextTestMatrix):
                 "executionParams": {
                     "selector": selector,
                     "runConfigData": merge_dicts(
-                        csv_hello_world_solids_config(), {"storage": {"filesystem": {}}}
+                        csv_hello_world_solids_config(),
+                        {"intermediate_storage": {"filesystem": {}}},
                     ),
                     "mode": "default",
                 }
@@ -351,8 +524,8 @@ class TestExecutePipeline(ExecutingGraphQLContextTestMatrix):
 
         assert first_event_of_type(logs, "PipelineStartEvent")["level"] == "DEBUG"
 
-        sum_solid_output = get_step_output_event(logs, "sum_solid.compute")
-        assert sum_solid_output["stepKey"] == "sum_solid.compute"
+        sum_solid_output = get_step_output_event(logs, "sum_solid")
+        assert sum_solid_output["stepKey"] == "sum_solid"
         assert sum_solid_output["outputName"] == "result"
 
     def test_basic_start_pipeline_execution_with_tags(self, graphql_context):
@@ -457,3 +630,26 @@ def get_step_output_event(logs, step_key, output_name="result"):
             return log
 
     return None
+
+
+class TestExecutePipelineReadonlyFailure(ReadonlyGraphQLContextTestMatrix):
+    def test_start_pipeline_execution_readonly_failure(self, graphql_context):
+        assert graphql_context.read_only == True
+
+        selector = infer_pipeline_selector(graphql_context, "csv_hello_world")
+        result = execute_dagster_graphql(
+            graphql_context,
+            LAUNCH_PIPELINE_EXECUTION_MUTATION,
+            variables={
+                "executionParams": {
+                    "selector": selector,
+                    "runConfigData": csv_hello_world_solids_config(),
+                    "mode": "default",
+                }
+            },
+        )
+
+        assert not result.errors
+        assert result.data
+
+        assert result.data["launchPipelineExecution"]["__typename"] == "ReadOnlyError"
