@@ -1,6 +1,6 @@
 import pendulum
-from dagster.core.definitions.job import JobType
-from dagster.core.host_representation import RepositoryLocationHandleManager
+from dagster.cli.workspace.dynamic_workspace import DynamicWorkspace
+from dagster.core.definitions.run_request import JobType
 from dagster.core.host_representation.grpc_server_registry import ProcessGrpcServerRegistry
 from dagster.core.scheduler.job import JobState, JobStatus
 from dagster.daemon import get_default_daemon_logger
@@ -15,7 +15,7 @@ from dagster_graphql.test.utils import (
 
 from .graphql_context_test_suite import (
     ExecutingGraphQLContextTestMatrix,
-    ReadonlyGraphQLContextTestMatrix,
+    NonLaunchableGraphQLContextTestMatrix,
 )
 
 GET_SENSORS_QUERY = """
@@ -161,7 +161,7 @@ mutation($jobOriginId: String!) {
 """
 
 
-class TestSensors(ReadonlyGraphQLContextTestMatrix):
+class TestSensors(NonLaunchableGraphQLContextTestMatrix):
     def test_get_sensors(self, graphql_context, snapshot):
         selector = infer_repository_selector(graphql_context)
         result = execute_dagster_graphql(
@@ -271,11 +271,11 @@ def test_sensor_next_ticks(graphql_context):
 
 
 def _create_tick(instance):
-    with ProcessGrpcServerRegistry(wait_for_processes_on_exit=True) as grpc_server_registry:
-        with RepositoryLocationHandleManager(grpc_server_registry) as handle_manager:
+    with ProcessGrpcServerRegistry() as grpc_server_registry:
+        with DynamicWorkspace(grpc_server_registry) as workspace:
             list(
                 execute_sensor_iteration(
-                    instance, get_default_daemon_logger("SensorDaemon"), handle_manager
+                    instance, get_default_daemon_logger("SensorDaemon"), workspace
                 )
             )
 
